@@ -7,7 +7,7 @@
 gSOAP XML Web services tools
 Copyright (C) 2000-2014, Robert van Engelen, Genivia Inc. All Rights Reserved.
 This software is released under one of the following licenses:
-GPL or Genivia's license for commercial use.
+GPL.
 --------------------------------------------------------------------------------
 GPL license.
 
@@ -135,6 +135,7 @@ class wsdl__ioput
         @xsd__QName                     element;                // WSDL 2.0
         @xsd__anyURI                    wsa__Action;
         @xsd__anyURI                    wsam__Action;
+        @xsd__anyURI                    wsaw__Action;
         xsd__string                     documentation;          // <wsdl:documentation>?
         wsp__Policy                     *wsp__Policy_;          // <wsp:Policy>?
         wsp__PolicyReference            *wsp__PolicyReference_; // <wsp:PolicyReference>?
@@ -160,6 +161,7 @@ class wsdl__fault
         @xsd__QName                     element;                // WSDL 2.0
         @xsd__anyURI                    wsa__Action;
         @xsd__anyURI                    wsam__Action;
+        @xsd__anyURI                    wsaw__Action;
         xsd__string                     documentation;          // <wsdl:documentation>?
         wsp__Policy                     *wsp__Policy_;          // <wsp:Policy>?
         wsp__PolicyReference            *wsp__PolicyReference_; // <wsp:PolicyReference>?
@@ -308,6 +310,7 @@ class wsdl__binding
         std::vector<wsp__PolicyReference> wsp__PolicyReference_;// <wsp:PolicyReference>*
         soap__binding                   *soap__binding_;        // <soap:binding>?
         http__binding                   *http__binding_;        // <http:binding>?
+        xsd__string                     *wsaw__UsingAddressing; // <wsaw:UsingAddressing wsdl:required="true">?
         std::vector<wsoap__module>      wsoap__module_;         // <wsoap:module>* WSDL 2.0
         std::vector<wsdl__ext_fault>    fault;                  // <wsdl:fault>* WSDL 2.0
         std::vector<wsdl__ext_operation> operation;             // <wsdl:operation>*
@@ -393,13 +396,13 @@ class wsdl__definitions
   public:
                                         wsdl__definitions();
                                         wsdl__definitions(struct soap*);
-                                        wsdl__definitions(struct soap*, const char*, const char*);
+                                        wsdl__definitions(struct soap*, const char*, const char*, const char*);
         virtual                         ~wsdl__definitions();
         int                             get(struct soap*);      // getter is triggered after parsing
         int                             preprocess();
         int                             traverse();
         int                             read(int, char**);
-        int                             read(const char *cwd, const char*);
+        int                             read(const char*, const char*, const char*);
         const char*                     sourceLocation();
         char*                           absoluteLocation(const char *loc) const;
         int                             error();
@@ -420,9 +423,9 @@ class wsdl__definitions
         void                            appPtr(wadl__application*);
         wadl__application*              appPtr() const;
         void                            mark();
-        friend ostream&                 operator<<(ostream&, const wsdl__definitions&);
-        friend istream&                 operator>>(istream&, wsdl__definitions&);
+        friend std::ostream&            operator<<(std::ostream&, const wsdl__definitions&);
+        friend std::istream&            operator>>(std::istream&, wsdl__definitions&);
 };
 
-extern ostream &operator<<(ostream &o, const wsdl__definitions &e);
-extern istream &operator>>(istream &i, wsdl__definitions &e);
+extern std::ostream &operator<<(std::ostream &o, const wsdl__definitions &e);
+extern std::istream &operator>>(std::istream &i, wsdl__definitions &e);
